"""
Provides systembridgedata version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update systembridgedata` to change this file.

from incremental import Version

__version__ = Version("systembridgedata", 5, 0, 0, dev=1)
__all__ = ["__version__"]
